import pymel.core as pm
from PySide2.QtWidgets import QApplication, QDialog, QVBoxLayout, QCheckBox, QPushButton, QLabel, QMessageBox, QRadioButton

class AxisSelectionDialog(QDialog):
    def __init__(self, parent=None):
        super(AxisSelectionDialog, self).__init__(parent)
        self.setWindowTitle("EasyMeasure")
        self.selected_axes = ["x", "y", "z"]
        self.place_on_edges = True
        
        self.initUI()
        
    def initUI(self):
        layout = QVBoxLayout()

        description_label_1 = QLabel("1. Select the object to measure in the viewport.<br><br>2. Select the axes to measure:")
        layout.addWidget(description_label_1)
        
        self.checkbox_x = QCheckBox("X")
        self.checkbox_x.setChecked(True)
        self.checkbox_x.stateChanged.connect(self.updateSelectedAxes)
        layout.addWidget(self.checkbox_x)
        
        self.checkbox_y = QCheckBox("Y")
        self.checkbox_y.setChecked(True)
        self.checkbox_y.stateChanged.connect(self.updateSelectedAxes)
        layout.addWidget(self.checkbox_y)
        
        self.checkbox_z = QCheckBox("Z")
        self.checkbox_z.setChecked(True)
        self.checkbox_z.stateChanged.connect(self.updateSelectedAxes)
        layout.addWidget(self.checkbox_z)

        description_label_2 = QLabel("3. Place the measures on the edges or center<br>of the bounding box?")
        layout.addWidget(description_label_2)
        
        self.radio_edges = QRadioButton("Edges")
        self.radio_edges.setChecked(True)
        self.radio_edges.toggled.connect(self.updateMeasurementMode)
        layout.addWidget(self.radio_edges)
        
        self.radio_center = QRadioButton("Center")
        self.radio_center.setChecked(False)
        self.radio_center.toggled.connect(self.updateMeasurementMode)
        layout.addWidget(self.radio_center)
        
        self.btn_measure = QPushButton("Measure")
        self.btn_measure.clicked.connect(self.measureDimensions)
        layout.addWidget(self.btn_measure)
        
        self.setLayout(layout)
        
    def updateSelectedAxes(self):
        self.selected_axes = []
        if self.checkbox_x.isChecked():
            self.selected_axes.append("x")
        if self.checkbox_y.isChecked():
            self.selected_axes.append("y")
        if self.checkbox_z.isChecked():
            self.selected_axes.append("z")

    def updateMeasurementMode(self):
        self.place_on_edges = self.radio_edges.isChecked()
    
    def measureDimensions(self):
        selected_objects = pm.ls(selection=True)
        if not selected_objects:
            # If no object is selected, show an error message
            QMessageBox.critical(self, "Error", "No object selected. Please select an object to measure.")
            return
    
        selected_object = selected_objects[0]
        dimensions = measure_dimensions(selected_object, self.selected_axes, self.place_on_edges)
        self.accept()

def measure_dimensions(selected_object, axes, place_on_edges):
    # Get bounding box of the selected object
    bbox = selected_object.getBoundingBox()
    
    # Create distance dimensions
    dimensions = []
    for axis in axes:
        axis_name = {"x": "Width", "y": "Height", "z": "Depth"}[axis]
        
        # Calculate start and end points
        start_point = pm.datatypes.Vector()
        end_point = pm.datatypes.Vector()
        if place_on_edges:
            # Place measurements on edges
            if axis == "x":
                start_point.x = bbox.min().x
                end_point.x = bbox.max().x
                start_point.y = end_point.y = (bbox.max().y)
                start_point.z = end_point.z = (bbox.max().z)
            elif axis == "y":
                start_point.y = bbox.min().y
                end_point.y = bbox.max().y
                start_point.x = end_point.x = (bbox.max().x)
                start_point.z = end_point.z = (bbox.max().z)
            elif axis == "z":
                start_point.z = bbox.min().z
                end_point.z = bbox.max().z
                start_point.x = end_point.x = (bbox.max().x)
                start_point.y = end_point.y = (bbox.max().y)
        else:
            # Place measurements in the center
            if axis == "x":
                start_point.x = bbox.min().x
                end_point.x = bbox.max().x
                start_point.y = end_point.y = (bbox.max().y + bbox.min().y) / 2
                start_point.z = end_point.z = (bbox.max().z + bbox.min().z) / 2
            elif axis == "y":
                start_point.y = bbox.min().y
                end_point.y = bbox.max().y
                start_point.x = end_point.x = (bbox.max().x + bbox.min().x) / 2
                start_point.z = end_point.z = (bbox.max().z + bbox.min().z) / 2
            elif axis == "z":
                start_point.z = bbox.min().z
                end_point.z = bbox.max().z
                start_point.x = end_point.x = (bbox.max().x + bbox.min().x) / 2
                start_point.y = end_point.y = (bbox.max().y + bbox.min().y) / 2
        
        # Create dimension node
        dimension_node = pm.distanceDimension(sp=start_point, ep=end_point)
        
        # Get the locator nodes created by the dimension node
        start_locator = pm.listConnections(dimension_node.startPoint, s=True, d=False)[0]
        end_locator = pm.listConnections(dimension_node.endPoint, s=True, d=False)[0]
        
        # Parent locators under the dimension node
        pm.parent(start_locator, end_locator, dimension_node)
        
        dimensions.append((dimension_node, axis_name))
    
    return dimensions

# Create Qt application
app = QApplication.instance()
if not app:
    app = QApplication([])
    
# Create and show the dialog
dialog = AxisSelectionDialog()
dialog.show()